/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : spi_flash.c
* Author             : MCD Application Team
* Date First Issued  : 02/05/2007
* Description        : This file provides a set of functions needed to manage the
*                      communication between SPI peripheral and SPI M25P64 FLASH.
********************************************************************************
* History:
* 05/21/2007: V0.3
* 04/02/2007: V0.2
* 02/05/2007: V0.1
********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "spi_rf.h"

/* Private typedef -----------------------------------------------------------*/




/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : SPI_FLASH_Init
* Description    : Initializes the peripherals used by the SPI FLASH driver.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void SPI_RF_Init(void)
{
  SPI_InitTypeDef  SPI_InitStructure;
    GPIO_InitTypeDef GPIO_InitStructure;

    SPI_I2S_DeInit(SPI2);

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE );
    //SPI2??APB1?,????    
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);
       
    /* Configure SPI2 pins: SCK, MISO and MOSI */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;      //????SPI2_CLK?SPI3_MISO?SPI4_MOSI
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;  //??????
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    
    /* SPI1 configuration */
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;  //??SPI???????????:SPI??????????
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;        //??SPI????:????SPI
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;        //??SPI?????:SPI????8????
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;//SPI_CPOL_High;        //??????????:?????
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;//SPI_CPHA_2Edge;    //???????????
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;        //NSS?????(NSS??)????(??SSI?)??:??NSS???SSI???
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4;        //??????????:????????256
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;    //???????MSB???LSB???:?????MSB???
    SPI_InitStructure.SPI_CRCPolynomial = 7;    //CRC???????
    SPI_Init(SPI2, &SPI_InitStructure);  //??SPI_InitStruct???????????SPIx???
    
    /* Enable SPI1  */
    SPI_Cmd(SPI2, ENABLE); //??SPI??
	
	
  
  
     
}

/***********************************************
 :	Nrf24l01_InterFace()
 :	MCUNrf24l01Ľӿ
 :	/
 :	/
 :	/
***********************************************/
void    si4432_InterFace(void)
{

		GPIO_InitTypeDef GPIO_InitStructure;
	
	  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE );

         /* Configure PB11 RF_SDN as Output push-pull -------------------------------*/
        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
        GPIO_Init(GPIOB, &GPIO_InitStructure);

		/* Configure PB12 RF_Csn as Output push-pull -------------------------------*/
		    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
        GPIO_Init(GPIOB, &GPIO_InitStructure);

		/* Configure PB10 RF_IRQ as input */
        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
        GPIO_Init(GPIOB, &GPIO_InitStructure);

		//SPI_RF_CE_LOW();
		//SPI_RF_CS_HIGH();

	    /*
        iRF_CE      = negative;
        
        iSPI_Csn    = positive;
        iSPI_Sck    = negative;
        
        direction_CE     = direction_output;
        direction_Csn    = direction_output;
        direction_Sck    = direction_output;
        direction_Mosi   = direction_output;
        direction_Miso   = direction_input;
		*/
}

/*******************************************************************************
* Function Name  : SPI_RF_SendByte
* Description    : Sends a byte through the SPI interface and return the byte 
*                  received from the SPI bus.
* Input          : byte : byte to send.
* Output         : None
* Return         : The value of the received byte.
*******************************************************************************/
u8 SPI_RF_SendByte(u8 byte)
{
  /* Loop while DR register in not emplty */
  while(SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_TXE) == RESET);

  /* Send byte through the SPI1 peripheral */
  SPI_I2S_SendData(SPI2, byte);
	

  /* Wait to receive a byte */
  while(SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_RXNE) == RESET);

  /* Return the byte read from the SPI bus */
  return SPI_I2S_ReceiveData(SPI2);	
	
}

/*******************************************************************************
* Function Name  : SPI_RF_ReadByte
* Description    : Reads a byte from the SPI Flash.
*                  This function must be used only if the Start_Read_Sequence
*                  function has been previously called.
* Input          : None
* Output         : None
* Return         : Byte Read from the SPI Flash.
*******************************************************************************/
u8 SPI_RF_ReadByte(void)
{
  return (SPI_RF_SendByte(0X55));
}

 



/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME: void SpiWriteRegister(U8 reg, U8 value)
  +
  + DESCRIPTION:   This function writes the registers 
  + 					
  + INPUT:			 U8 reg - register address   
  +					 U8 value - value write to register	
  +
  + RETURN:        None
  +
  + NOTES:         Write uses a Double buffered transfer
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void SpiWriteRegister (u8 reg, u8 value)
{
   //uchar i;
   uchar Temp_byte;
   
   SPI_RF_CS_LOW();       //NSS = 0;                              
   Temp_byte = (reg|0x80);				//write data into the SPI register
	
	//--------------------------
	
	 SPI_RF_SendByte(Temp_byte);
		/*
        for(i = 0x00; i < 0x08; i++)
        {
            if(Temp_byte&0x80)
                SDI    = 1;
            else
                SDI    = 0;
			
            SCK         = 1;
            SCK         = 0;            
            Temp_byte <<= 0x01;
        }
		*/
	
	
	//--------------------------
		Temp_byte = value;
		SPI_RF_SendByte(Temp_byte);
		/*	
	    for(i = 0x00; i < 0x08; i++)
        {
            if(Temp_byte&0x80)
                SDI    = 1;
            else
                SDI    = 0;
			
            SCK         = 1;
            SCK         = 0;            
            Temp_byte           <<= 0x01;
        }
		*/

	SPI_RF_CS_HIGH();//NSS = 1;    
                        
  
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME: U8 SpiReadRegister(U8 reg)
  +
  + DESCRIPTION:   This function reads the registers 
  + 					
  + INPUT:			 U8 reg - register address   
  +
  + RETURN:        SPI1DAT - the register content 
  +
  + NOTES:         none
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
U8 SpiReadRegister (u8 reg)
{
    //uchar i;
    uchar Temp_byte;
	uchar Result;
	
	SPI_RF_CS_LOW();       //NSS = 0; 
	Temp_byte = reg;				//write data into the SPI register
	SPI_RF_SendByte(Temp_byte);
	Result = SPI_RF_ReadByte();

	/*
	//--------------------------
	

        for(i = 0x00; i < 0x08; i++)
        {
            if(Temp_byte&0x80)
                SDI    = 1;
            else
                SDI    = 0;
			
            SCK         = 1;
            SCK         = 0;            
            Temp_byte           <<= 0x01;
        }
		
		Result=0;
		
  	    for(i=0;i<8;i++)
  	    {                    //read fifo data byte
  	 	     Result=Result<<1;
  	 	     SCK=1;
  	 	     
  	 	    if(SDO)
  	 	    {
  	 		        Result|=1;
  	 	    }
  	 	
  	 	    SCK=0;
  	 	     
  	    }
  	
	  */  
	
	
	
  
	
	SPI_RF_CS_HIGH();//NSS = 1;                            

	
   return(Result);
   

}

void init_4432(void)
{

	u8 temp8;
	u16 delay;
	u8 ItStatus1;
	u8 ItStatus2;
    /* ======================================================== *
		*						Initialize the Si443x ISM chip				* 
	* ======================================================== */
	si4432_InterFace(); 		
  SPI_RF_Init();
	//Turn on the radio by pulling down the PWRDN pin
	SPI_RF_SDN_HIGH();  //SDN = 1;
	for (temp8=0;temp8<150;temp8++)
	{
		for(delay=0;delay<10000;delay++);
	}	 
	
	SPI_RF_SDN_LOW();  //SDN = 0;
	//SPI_RF_SDN_HIGH();
	//Wait at least 15ms befory any initialization SPI commands are sent to the radio
	// (wait for the power on reset sequence) 
	for (temp8=0;temp8<150;temp8++)
	{
		for(delay=0;delay<10000;delay++);
	}	 
	//read interrupt status registers to clear the interrupt flags and release NIRQ pin
	
	ItStatus1 = SpiReadRegister(0x03);													//read the Interrupt Status1 register
	ItStatus2 = SpiReadRegister(0x04);													//read the Interrupt Status2 register
    
	//SW reset   
    SpiWriteRegister(0x07, 0x80);															//write 0x80 to the Operating & Function Control1 register 
	
	//wait for POR interrupt from the radio (while the nIRQ pin is high)
	while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_10) != 0x00);  //while ( NIRQ == 1);  
	//read interrupt status registers to clear the interrupt flags and release NIRQ pin
	ItStatus1 = SpiReadRegister(0x03);													//read the Interrupt Status1 register
	ItStatus2 = SpiReadRegister(0x04);													//read the Interrupt Status2 register
     
	//wait for chip ready interrupt from the radio (while the nIRQ pin is high)
	while(GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_10) != 0x00);//while ( NIRQ == 1);  
	//read interrupt status registers to clear the interrupt flags and release NIRQ pin
	ItStatus1 = SpiReadRegister(0x03);													//read the Interrupt Status1 register
	ItStatus2 = SpiReadRegister(0x04);													//read the Interrupt Status2 register
						
							/*set the physical parameters*/
	//set the center frequency to 915 MHz
	//SpiWriteRegister(0x75, 0x75);															//write 0x75 to the Frequency Band Select register             
	//SpiWriteRegister(0x76, 0xBB);															//write 0xBB to the Nominal Carrier Frequency1 register
	//SpiWriteRegister(0x77, 0x80); 
 														//write 0x80 to the Nominal Carrier Frequency0 register
	
	SpiWriteRegister(0x75, 0x53);															//write 0x75 to the Frequency Band Select register             
	SpiWriteRegister(0x76, 0x64);															//write 0xBB to the Nominal Carrier Frequency1 register
	SpiWriteRegister(0x77, 0x00); 
	
	
	//set the desired TX data rate (9.6kbps)
	//SpiWriteRegister(0x6E, 0x4E);															//write 0x4E to the TXDataRate 1 register
	//SpiWriteRegister(0x6F, 0xA5);															//write 0xA5 to the TXDataRate 0 register
	//SpiWriteRegister(0x70, 0x2C);															//write 0x2C to the Modulation Mode Control 1 register
    
	
	//set the desired TX data rate (1.2kbps)
	SpiWriteRegister(0x2a, 0x14);	
	SpiWriteRegister(0x6E, 0x09);															//write 0x4E to the TXDataRate 1 register
	SpiWriteRegister(0x6F, 0xd5);															//write 0xA5 to the TXDataRate 0 register
	SpiWriteRegister(0x70, 0x2C);	
	
													

	
	//set the Tx deviation register (+-45kHz)
	//SpiWriteRegister(0x72, 0x48);	//(9.6kbps)														//write 0x48 to the Frequency Deviation register 
    SpiWriteRegister(0x72, 0x38);	//(1.2kbps)	

								/*set the modem parameters according to the exel calculator(parameters: 9.6 kbps, deviation: 45 kHz, channel filter BW: 102.2 kHz*/
	SpiWriteRegister(0x1C, 0x1b);															//write 0x1E to the IF Filter Bandwidth register		
	SpiWriteRegister(0x20, 0x83);															//write 0xD0 to the Clock Recovery Oversampling Ratio register		
	SpiWriteRegister(0x21, 0xc0);															//write 0x00 to the Clock Recovery Offset 2 register		
	SpiWriteRegister(0x22, 0x13);															//write 0x9D to the Clock Recovery Offset 1 register		
	SpiWriteRegister(0x23, 0xa9);															//write 0x49 to the Clock Recovery Offset 0 register		
	SpiWriteRegister(0x24, 0x00);															//write 0x00 to the Clock Recovery Timing Loop Gain 1 register		
	SpiWriteRegister(0x25, 0x03);															//write 0x24 to the Clock Recovery Timing Loop Gain 0 register		
	SpiWriteRegister(0x1D, 0x40);															//write 0x40 to the AFC Loop Gearshift Override register		
	SpiWriteRegister(0x1E, 0x0A);															//write 0x0A to the AFC Timing Control register		
	SpiWriteRegister(0x2A, 0x14);															//write 0x20 to the AFC Limiter register		
						
							/*set the packet structure and the modulation type*/
	//set the preamble length to 5bytes 
	SpiWriteRegister(0x34, 0x0A);															//write 0x0A to the Preamble Length register
	//set preamble detection threshold to 20bits
	SpiWriteRegister(0x35, 0x2A); 														//write 0x2A to the Preamble Detection Control  register

	//Disable header bytes; set variable packet length (the length of the payload is defined by the
	//received packet length field of the packet); set the synch word to two bytes long
	SpiWriteRegister(0x33, 0x02);															//write 0x02 to the Header Control2 register    
	
	//Set the sync word pattern to 0x2DD4
	SpiWriteRegister(0x36, 0x2D);															//write 0x2D to the Sync Word 3 register
	SpiWriteRegister(0x37, 0xD4);															//write 0xD4 to the Sync Word 2 register

	//enable the TX & RX packet handler and CRC-16 (IBM) check
	SpiWriteRegister(0x30, 0x8D);															//write 0x8D to the Data Access Control register
	//Disable the receive header filters
  SpiWriteRegister(0x32, 0x00 );														//write 0x00 to the Header Control1 register            
	//enable FIFO mode and GFSK modulation
	SpiWriteRegister(0x71, 0x63);															//write 0x63 to the Modulation Mode Control 2 register

											/*set the GPIO's according to the RF switch */
  SpiWriteRegister(0x0C, 0x12);															//write 0x12 to the GPIO1 Configuration(set the TX state)
	SpiWriteRegister(0x0b, 0x15);															//write 0x15 to the GPIO2 Configuration(set the RX state) 

											/*set the non-default Si443x registers*/
	//set  cap. bank
	SpiWriteRegister(0x09, 0xD7);															//write 0xD7 to the Crystal Oscillator Load Capacitance register
	// Set AGC Override1 Register
	SpiWriteRegister(0x69, 0x60);															//write 0x60 to the AGC Override1 register	

	//set tx power 20dbm max
	//SpiWriteRegister(0x6d, 0x1f);	 
	//SpiWriteRegister(0x6d, 0x1e);	
	SpiWriteRegister(0x6d, 0x1f);	
	/*enable receiver chain*/
	SpiWriteRegister(0x07, 0x05);															//write 0x05 to the Operating Function Control 1 register
	//Enable two interrupts: 
	// a) one which shows that a valid packet received: 'ipkval'
	// b) second shows if the packet received with incorrect CRC: 'icrcerror' 
	SpiWriteRegister(0x05, 0x03); 														//write 0x03 to the Interrupt Enable 1 register
	SpiWriteRegister(0x06, 0x00); 														//write 0x00 to the Interrupt Enable 2 register
	//read interrupt status registers to release all pending interrupts
	ItStatus1 = SpiReadRegister(0x03);													//read the Interrupt Status1 register
	ItStatus2 = SpiReadRegister(0x04);
	
	/*enable receiver chain*/	    
		//SpiWriteRegister(0x07, 0x05);	
		
		//SpiWriteRegister(0x0d, 0xf4);	
}

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
